@@ -29,6 +29,9 @@ DEFAULT_DT = datetime.datetime.now().replace(day=1, hour=0, minute=0, second=0,
 class BaseModel:
     """The base model class all models should inherit from. Provides fetch and other utility functionalities.
 
+    :_endpoint_name: The endpoint to use in the api
+    :_nested_name:  The name of that will appear in nested results. "Agencies" and the such.
+
     =========  ===========
     Operation  Description
     ---------  -----------
@@ -36,20 +39,19 @@ class BaseModel:
     =========  ===========
     """
 
-    def __init__(self, endpoint: str, param_translations: dict, nested_name: str, api_instance: Api, proper_name: str):
+    _endpoint_name = ""
+    _nested_name = ""
+
+    def __init__(self, param_translations: dict, api_instance: Api, proper_name: str):
         """
         All launchlibrary models should inherit from this class. Contains utility and fetch functions.
 
-        :param endpoint: The endpoint to use in the api
         :param param_translations:  Translations from API names to pythonic names.
-        :param nested_name:  The name of that will appear in nested results. "Agencies" and the such.
         :param api_instance:  An instance of the Api class.
         :param proper_name:  The proper name for use in __repr__
         """
         # param translations serves both for pythonic translations and default param values
         self._param_translations = param_translations
-        self.endpoint = endpoint
-        self.nested_name = nested_name
         self.api_instance = api_instance
         self.proper_name = proper_name
         self.param_names = self._param_translations.keys()
@@ -64,11 +66,25 @@ class BaseModel:
         """
 
         kwargs = utils.sanitize_input(kwargs)
-        cls_init = cls(api_instance)
-        json_object = api_instance._send_message(cls_init.endpoint, kwargs)
-        classes = []
 
-        for entry in json_object.get(cls_init.nested_name, []):
+        json_object = api_instance.send_message(cls._endpoint_name, kwargs)
+
+        classes = cls._create_classes(api_instance, json_object)
+
+        return classes
+
+    @classmethod
+    def _create_classes(cls, api_instance: Api, json_object) -> list:
+        """
+        Creates the required classes from the json object.
+
+        :param api_instance:
+        :param json_object:
+        :return:
+        """
+
+        classes = []
+        for entry in json_object.get(cls._nested_name, []):
             cls_init = cls(api_instance)
             cls_init._set_params_json(entry)
             cls_init._postprocess()
@@ -131,6 +147,8 @@ class BaseModel:
 
 class AgencyType(BaseModel):
     """A class representing an agency type object."""
+    _nested_name = "types"
+    _endpoint_name = "agencytype"
 
     def __init__(self, api_instance: Api):
         param_translations = dict(id="id", name="name", changed="changed")
@@ -140,15 +158,16 @@ class AgencyType(BaseModel):
         self.changed = None
 
         proper_name = self.__class__.__name__
-        nested_name = "types"
-        endpoint_name = "agencytype"
 
-        super().__init__(endpoint_name, param_translations, nested_name, api_instance, proper_name)
+        super().__init__(param_translations, api_instance, proper_name)
 
 
 class Agency(BaseModel):
     """A class representing an agency object."""
 
+    _nested_name = "agencies"
+    _endpoint_name = "agency"
+
     def __init__(self, api_instance: Api):
         param_translations = dict(id="id", name="name", abbrev="abbrev", type="type", country_code="countryCode"
                                   , wiki_url="wikiURL", info_urls="infoURLs", is_lsp="islsp", changed="changed")
@@ -164,14 +183,11 @@ class Agency(BaseModel):
         self.changed = None
 
         proper_name = self.__class__.__name__
-        nested_name = "agencies"
-        endpoint_name = "agency"
 
-        super().__init__(endpoint_name, param_translations, nested_name, api_instance, proper_name)
+        super().__init__(param_translations, api_instance, proper_name)
 
     @lru_cache(maxsize=None)
     def get_type(self) -> AgencyType:
-        """Returns the name of the agency type."""
         if self.type:
             agency_type = AgencyType.fetch(self.api_instance, id=self.type)
         else:
@@ -184,6 +200,9 @@ class Agency(BaseModel):
 class LaunchStatus(BaseModel):
     """A class representing a launch status object."""
 
+    _nested_name = "types"
+    _endpoint_name = "launchstatus"
+
     def __init__(self, api_instance: Api):
         param_translations = dict(id="id", name="name", description="description", changed="changed")
 
@@ -193,10 +212,8 @@ class LaunchStatus(BaseModel):
         self.changed = None
 
         proper_name = self.__class__.__name__
-        nested_name = "types"
-        endpoint_name = "launchstatus"
 
-        super().__init__(endpoint_name, param_translations, nested_name, api_instance, proper_name)
+        super().__init__(param_translations, api_instance, proper_name)
 
 
 class Launch(BaseModel):
@@ -211,13 +228,16 @@ class Launch(BaseModel):
     x > y      Checks if launch x occurs before launch y.
     =========  ==========="""
 
+    _nested_name = "launches"
+    _endpoint_name = "launch"
+
     def __init__(self, api_instance: Api):
         self.datetime_conversions = dict()
         param_translations = dict(id="id", name="name", tbddate="tbddate", tbdtime="tbdtime", status="status"
                                   , inhold="inhold", windowstart="isostart", windowend="isoend",
                                   net="isonet", info_urls="infoURLs",
                                   vid_urls="vidURLs", holdreason="holdreason", failreason="failreason",
-                                  probability="probability", hashtag="hashtag", _lsp="lsp", changed="changed",
+                                  probability="probability", hashtag="hashtag", agency="lsp", changed="changed",
                                   location="location", rocket="rocket", missions="missions")
 
         self.id = None
@@ -242,10 +262,8 @@ class Launch(BaseModel):
         self.missions = None
 
         proper_name = self.__class__.__name__
-        nested_name = "launches"
-        endpoint_name = "launch"
 
-        super().__init__(endpoint_name, param_translations, nested_name, api_instance, proper_name)
+        super().__init__(param_translations, api_instance, proper_name)
 
     @classmethod
     def next(cls, api_instance: Api, num: int) -> List["Launch"]:
@@ -256,21 +274,6 @@ class Launch(BaseModel):
         """
         return cls.fetch(api_instance, next=num, status=1)
 
-    @lru_cache(maxsize=None)
-    def get_agency(self) -> Agency:
-        """Gets the Agency model of the launch service provider either from the response or from the server."""
-        if self._lsp:
-            if self.api_instance.mode == "verbose" and isinstance(self._lsp, dict):
-                lsp = Agency.init_from_json(self.api_instance, self._lsp)
-            else:
-                lsp = Agency.fetch(self.api_instance, id=self._lsp)
-                if len(lsp) > 0:
-                    lsp = lsp[0]
-        else:
-            lsp = Agency.init_from_json(self.api_instance, {})  # Init an empty model to prevent errors
-
-        return lsp
-
     @lru_cache(maxsize=None)
     def get_status(self) -> LaunchStatus:
         """Returns the LaunchStatus model for the corresponding status."""
@@ -292,14 +295,19 @@ class Launch(BaseModel):
                 # The string might not contain a date, so we'll need to handle it with an empty datetime object.
                 setattr(self, time_name, DEFAULT_DT)
 
-    def __lt__(self, other: "Launch") -> bool: return self.net < other.net
+    def __lt__(self, other: "Launch") -> bool:
+        return self.net < other.net
 
-    def __gt__(self, other: "Launch") -> bool: return self.net > other.net
+    def __gt__(self, other: "Launch") -> bool:
+        return self.net > other.net
 
 
 class Pad(BaseModel):
     """A class representing a pad object."""
 
+    _nested_name = "pads"
+    _endpoint_name = "pad"
+
     def __init__(self, api_instance: Api):
         param_translations = dict(id="id", name="name", pad_type="padType", latitude="latitude", longitude="longitude",
                                   map_url="mapURL", retired="retired", locationid="locationid", agencies="agencies",
@@ -318,15 +326,16 @@ class Pad(BaseModel):
         self.info_urls = None
 
         proper_name = self.__class__.__name__
-        nested_name = "pads"
-        endpoint_name = "pad"
 
-        super().__init__(endpoint_name, param_translations, nested_name, api_instance, proper_name)
+        super().__init__(param_translations, api_instance, proper_name)
 
 
 class Location(BaseModel):
     """A class representing a location object."""
 
+    _nested_name = "locations"
+    _endpoint_name = "location"
+
     def __init__(self, api_instance: Api):
         param_translations = dict(id="id", name="name", country_code="countrycode", wiki_url="wikiURL"
                                   , info_urls="infoURLs", pads="pads")  # pads might be included w/ launch endpoint
@@ -339,15 +348,16 @@ class Location(BaseModel):
         self.pads = None
 
         proper_name = self.__class__.__name__
-        nested_name = "pads"
-        endpoint_name = "pad"
 
-        super().__init__(endpoint_name, param_translations, nested_name, api_instance, proper_name)
+        super().__init__(param_translations, api_instance, proper_name)
 
 
 class RocketFamily(BaseModel):
     """A class representing a rocket family object."""
 
+    _nested_name = "RocketFamilies"
+    _endpoint_name = "rocketfamily"
+
     def __init__(self, api_instance: Api):
         param_translations = dict(id="id", name="name", agencies="agencies", changed="changed")
 
@@ -357,15 +367,16 @@ class RocketFamily(BaseModel):
         self.changed = None
 
         proper_name = self.__class__.__name__
-        nested_name = "RocketFamilies"
-        endpoint_name = "rocketfamily"
 
-        super().__init__(endpoint_name, param_translations, nested_name, api_instance, proper_name)
+        super().__init__(param_translations, api_instance, proper_name)
 
 
 class Rocket(BaseModel):
     """A class representing a rocket object."""
 
+    _nested_name = "rockets"
+    _endpoint_name = "rocket"
+
     def __init__(self, api_instance: Api):
         param_translations = dict(id="id", name="name", default_pads="defaultPads", family="family",
                                   wiki_url="wikiURL", info_urls="infoURLs", image_url="imageURL"
@@ -381,22 +392,21 @@ class Rocket(BaseModel):
         self.image_sizes = None
 
         proper_name = self.__class__.__name__
-        nested_name = "rockets"
-        endpoint_name = "rocket"
 
-        super().__init__(endpoint_name, param_translations, nested_name, api_instance, proper_name)
+        super().__init__(param_translations, api_instance, proper_name)
 
     @lru_cache(maxsize=None)
     def get_pads(self) -> List[Pad]:
         """Returns Pad type objects of the pads the rocket uses."""
+        pad_objs = []
         if self.default_pads:
             pad_objs = Pad.fetch(self.api_instance, id=self.default_pads)
-            return pad_objs if pad_objs else []  # Make sure to obey types
-        else:
-            return []
+
+        return pad_objs
 
 
 # putting it at the end to load the classes first
 MODEL_LIST_PLURAL = {"agencies": Agency, "pads": Pad, "locations": Location
     , "rockets": Rocket, "families": RocketFamily}
-MODEL_LIST_SINGULAR = {"agency": Agency, "pad": Pad, "location": Location, "rocket": Rocket, "family": RocketFamily}
+MODEL_LIST_SINGULAR = {"agency": Agency, "pad": Pad, "location": Location, "rocket": Rocket, "family": RocketFamily,
+                       "lsp": Agency}
